package com.winhc.bigdata.spark.implicits

import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.DataFrame
import org.apache.spark.sql.functions.col
import org.elasticsearch.spark._

/**
 * @Author: XuJiakai
 * @Date: 2020/7/11 16:25
 * @Description:
 */
object CompanyIndexSave2EsHelper {
  val companyIndexFields = Seq(
    "cid"
    , "name"
    , "history_names"
    , "current_cid"
    , "company_type" //公司类型

    , "credit_code" //统一信用代码
    , "reg_status" //公司状态
    , "area_code" // 行政区划码
    , "reg_location" //注册地址
    , "estiblish_time" // 成立日期(注册时间)
    , "lat" //公司纬度
    , "lng" //公司经度
    , "category_code" // 行业分类
    , "reg_capital" // 注册资本,展示类型
    , "reg_capital_amount" // 注册资本,数值类型
    , "phones" //电话
    , "emails" //邮箱
  )

  implicit class DataFrameEnhancer(df: DataFrame) {
    def companyIndexSave2Es(): Unit = {
      import df.sparkSession.implicits._
      df.select(companyIndexFields.map(column => col(column).cast("string")): _*)
        .rdd.map(r => {
        val map = companyIndexFields.map(f => {
          (f, r.getAs[String](f))
        }).toMap
        getEsDoc(map)
      })
        .saveToEsWithMeta("winhc-company-v5/company")
    }
  }

  case class Geo(lat: String, lon: String)

  case class CompanyName(show: String, value: String)

  case class CompanyDoc(
                         cname: CompanyName
                         , current_id: String
                         , history_name: Seq[CompanyName]
                         , company_type: String
                         , credit_code: String
                         , reg_status: String
                         //                         , geo: Geo
                         , province_code: String
                         , city_code: String
                         , county_code: String
                         , reg_location: String
                         , estiblish_time: String
                         , category_code: String
                         , reg_capital: String
                         , reg_capital_amount: String
                         , phones: Seq[String]
                         , emails: Seq[String]
                       )

  val pattern = "[^\\u4e00-\\u9fa50-9a-zA-Z]".r

  def getEsDoc(map: Map[String, String]): (String, CompanyDoc) = {
    val lat = map("lat")
    val lng = map("lng")
    var geo: String = null
    if (StringUtils.isNotBlank(lat) && StringUtils.isNotBlank(lng)) {
      geo = lat + "," + lng
    }

    val c = get_area_code(map("area_code"))
    val province_code = c._1
    val city_code = c._2
    val county_code = c._3
    val et = map("estiblish_time")
    var time = if (StringUtils.isNotBlank(et)){
      if (et.contains(" ")) {
        et.split(" ")(0)
      }else{
        et
      }
    }  else null


    val doc = CompanyDoc(
      cname = getCompanyName(map("name"))
      , current_id = map("current_cid")
      , history_name = getHistoryName(map("name"), map("history_names"))
      , company_type = map("company_type")
      , credit_code = map("credit_code")
      , reg_status = map("reg_status")
      //      , geo = Geo(lat = lng, lon = lat)
      , province_code = province_code
      , city_code = city_code
      , county_code = county_code
      , reg_location = map("reg_location")
      , estiblish_time = time
      , category_code = map("category_code")
      , reg_capital = map("reg_capital")
      , reg_capital_amount = map("reg_capital_amount")
      , phones = getSplit(map("phones"))
      , emails = getSplit(map("emails"))
    )
    (map("cid"), doc)
  }


  private def getCompanyName(name: String): CompanyName = {
    if (StringUtils.isEmpty(name)) null
    else {
      val value = pattern replaceAllIn(name, "")
      CompanyName(name, value)
    }
  }

  private def getHistoryName(cname: String, names: String): Seq[CompanyName] = {
    if (StringUtils.isEmpty(names)) {
      null
    } else {
      val res = getSplit(names)
        .filter(!cname.equals(_))
        .filter(StringUtils.isNoneEmpty(_))
        .map(getCompanyName)
      if (res.isEmpty) {
        null
      } else {
        res
      }
    }
  }

  private def get_area_code(code: String): (String, String, String) = {
    if (StringUtils.isNotBlank(code) && code.trim.length == 6) {
      val c = code.trim
      (c.substring(0, 2), c.substring(2, 4), c.substring(4, 6))
    } else {
      (null, null, null)
    }
  }

  private def getSplit(str: String): Seq[String] = {
    if (StringUtils.isNotBlank(str)) {
      str.split("\t;\t").filter(StringUtils.isNotBlank).toSet.toList
    } else {
      Seq.empty[String]
    }
  }

  private val DATE_TIME_FORMAT = "yyyy-MM-dd"

  private def validateDf(str: String): Boolean = try {
    if (StringUtils.isNotBlank(str)) {
      java.time.LocalDateTime.parse(str, java.time.format.DateTimeFormatter.ofPattern(DATE_TIME_FORMAT))
      true
    } else {
      false
    }
  } catch {
    case ex: java.time.format.DateTimeParseException => {
      false
    }
  }

  def main(args: Array[String]): Unit = {
    println(validateDf("2010-03-03 00:00:00"))
  }
}
