package com.winhc.bigdata.spark.model

import java.util.Date

import com.winhc.bigdata.calc.{DimScore, DimScoreV2}
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.broadcast.Broadcast
import org.apache.spark.sql.{Row, SparkSession}

import scala.annotation.meta.getter
import scala.collection.mutable

/**
 * 招投标
 * π
 */

object CompanyBidScore {

  val tabMapping: Map[String, (String, String, String, String)] =
    Map("ads_company_bid_list" -> ("1", "publish_time", "资产权益", "招投标") //招投标
    )

  def main(args: Array[String]): Unit = {

    val (sourceTable, flag, time, kind, project) = valid(args)

    var config = mutable.Map.empty[String, String]

    val spark: SparkSession = SparkUtils.InitEnv(this.getClass.getSimpleName, config)

    println(s"company ${this.getClass.getSimpleName} calc start! " + new Date().toString)

    new CompanyBidScore(spark, sourceTable, flag, time, kind, project).calc()

    println(s"company ${this.getClass.getSimpleName} calc end! " + new Date().toString)
    spark.stop()
  }

  def valid(args: Array[String]) = {
    if (args.length != 1) {
      println("请输入要计算的table!!!! ")
      sys.exit(-1)
    }
    val Array(sourceTable) = args

    val (flag, time, kind, project) = tabMapping.getOrElse(sourceTable, ("", "", "", ""))
    if (flag.isEmpty || time.isEmpty || kind.isEmpty || project.isEmpty) {
      println("输入表不存在!!!   ")
      sys.exit(-1)
    }
    (sourceTable, flag, time, kind, project)
  }
}

case class CompanyBidScore(s: SparkSession, sourceTable: String,
                           flag: String, time: String, kind: String, project: String
                          ) extends LoggingUtils {

  @(transient@getter) val spark: SparkSession = s

  import spark.implicits._

  def calc(): Unit = {

    val ods_company = "new_ods_company"
    val company_category = "const_company_category_code"

    //    //所属行业
    //    val code2Name: Broadcast[Map[String, String]] = spark.sparkContext.broadcast(sql(
    //      s"""
    //         |select category_code,category_str_big
    //         |from $company_category
    //      """.stripMargin).collect().map(r => {
    //      (r.getString(0), r.getString(1))
    //    }).toMap)
    //
    //    spark.udf.register("industry_name", (code: String) => {
    //      code2Name.value.getOrElse(code, null)
    //    })
    //
    //    val industry = sql(
    //      s"""
    //         |select category_code,cast(cid as string) as ncid,
    //         |       industry_name(category_code) AS industry_name
    //         |from $ods_company where cid is not null
    //         |""".stripMargin)
    //
    //    industry.show(100)
    //
    //
    //    industry.createOrReplaceTempView("t1")

    val industry2 = sql(
      s"""
         |select a.category_code,cast(a.cid as string) as ncid,
         |       b.category_str_big AS industry_name
         |from $ods_company  a
         |left join const_company_category_code b on a.category_code = b.category_code
         |where cid is not null
         |""".stripMargin)
    industry2.createOrReplaceTempView("t1")

    //    注意线上是否分区
    //     ds = '${BaseUtil.getPartion(sourceTable, spark)}' AND

    val df = sql(
      s"""
         |SELECT  *
         |FROM    (
         |        SELECT
         |                *
         |                ,COUNT(ncid) OVER(PARTITION BY ncid ) AS cnt1
         |                ,row_number() OVER(PARTITION BY ncid ORDER BY $time DESC ) AS num
         |        FROM    $sourceTable
         |        WHERE
         |
         |             ncid IS NOT NULL
         |        ) a
         |WHERE   num =1
         |""".stripMargin).createOrReplaceTempView("t2")
    //      .join(industry, Seq("ncid"), "left")
    //      .select("cid", "id", "cnt1", "industry_name", "ncid")

    val df2 = sql(
      """
        |select t2.*,t1.industry_name,category_code from t2 left join t1 on t2.ncid = t1.ncid
        |""".stripMargin)
    df2.show(100)

    df2.map(r => {
      trans(r, flag, kind, project)
    }).toDF("id", "cid", "kind", "kind_code", "project", "project_code", "type",
      "score", "total", "extraScore")
      .createOrReplaceTempView(s"${sourceTable}_tmp_view")


    sql(s"select * from ${sourceTable}_tmp_view").show(10)
    sql(s"insert overwrite table ${sourceTable}_score  select * from ${sourceTable}_tmp_view")
  }

  def trans(r: Row, flag: String, kind: String, prpject: String) = {
    val id = r.getAs[Long]("id")
    val cid = r.getAs[Long]("ncid").toString
    val cnt1 = r.getAs[Long]("cnt1")
    val industry_name = r.getAs[String]("industry_name")
    flag match {
      case "1" => tenderScore(id, cid, cnt1, kind, prpject, industry_name)
    }
  }

  //招投标
  def tenderScore(id: Long, cid: String, cnt1: Long, kind: String, project: String, industry_name: String) = {
    var score = 0f
    val total = 20f
    val extraScore = 0f
    var isBuilderCompany = false
    if (StringUtils.isNotBlank(industry_name)
      && industry_name.contains("建筑")) {
      isBuilderCompany = true
    }
    var ty = ""
    if (!isBuilderCompany) {
      if (cnt1 == 0) {
        score = 12f
        ty = "无（非建筑类企业）"
      } else {
        score = 20f
        ty = "有（非建筑类企业）"
      }
    } else {
      if (cnt1 == 0) {
        score = 5f
        ty = "无（建筑类企业）"
      } else if (cnt1 <= 3) {
        score = 12f
        ty = "招投标数量≤3"
      } else if (cnt1 <= 10) {
        score = 15f
        ty = "招投标数量＞3，≤10"
      } else if (cnt1 <= 50) {
        score = 17f
        ty = "招投标数量＞10，≤50"
      } else if (cnt1 <= 100) {
        score = 20f
        ty = "招投标数量＞50"
      } else {
        score = 25f
        ty = "招投标数量＞100，该项分数上加+5"
      }
    }

    (id, cid, kind, DimScoreV2.newsEventMap.get(kind), project, DimScoreV2.newsEventMap.get(project), ty,
      score, total, extraScore)
  }

}
